/*
   signal.c -- PIKT piktc & piktc_svc signal handling routines

   $Header: /home/cvs/ucgsb/pikt/src/piktc/signal.c,v 1.16 2000/01/12 19:25:14 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2000 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
*/

#define SIGNAL
#include "../lib/common.h"
#include "piktc.h"
#undef SIGNAL

#include <rpc/rpc.h>
#include "rpc.h"

struct sigaction sigchld_act;
struct sigaction sigterm_act;

void sig_chld(int);
void sig_term(int);

/* simplest version if don't care to know reason why child terminated */
/*
void
sig_init (void)
{

	sigchld_act.sa_handler = SIG_IGN;
	sigchld_act.sa_flags = SA_NOCLDWAIT;

	if (sigaction(SIGCHLD, &sigchld_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGCHLD");
	}
}
*/

/* initialize the signal handlers
 */
void
sig_init (void)
{

#ifdef SA_SIGINFO
	/* HMS: Why bother with sigaction if the function takes no args? */
	sigchld_act.sa_sigaction = (void *) sig_chld;
	sigchld_act.sa_flags = SA_SIGINFO
#ifdef SA_NOCLDWAIT
			       | SA_NOCLDWAIT
#endif
			       ;
#else
	sigchld_act.sa_handler =  sig_chld;
#endif
	sigemptyset(&sigchld_act.sa_mask);
	if (sigaction(SIGCHLD, &sigchld_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGCHLD");
	}

#ifdef SA_SIGINFO
	sigterm_act.sa_sigaction = (void *) sig_term;
	sigterm_act.sa_flags = 0
				| SA_SIGINFO
				;
#else
	sigterm_act.sa_handler =  sig_term;
#endif
	sigemptyset(&sigterm_act.sa_mask);
	if (sigaction(SIGTERM, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGTERM");
	}
	if (sigaction(SIGINT, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGINT");
	}
	if (sigaction(SIGQUIT, &sigterm_act, (struct sigaction *) NULL) < 0) {
		logexit(PIKTLOG, LOG_ERR, __FILE__, __LINE__, TRUE, "in sig_init(), sigaction() error for SIGQUIT");
	}
}

/* SIGCHLD handler
 */
void
sig_chld (int sig)
{

	/* log reason for any abnormal child (preprocess) terminations? */
	/* wait((int *)NULL); -- this caused no end of difficulties!! */
	return;
}

/* SIGTERM handler--on program termination, remove the piktc.lock file,
   also any files in the diffing & staging dirs; also, in the case of
   piktc, if in the middle of an execwait, send a free_execcmdout()
   request to the current client host
 */
void
sig_term (int sig)
{
	#ifndef PIKTC_SVC
	char cmd[MAXCMD+1];
	#endif

	#ifdef PIKTC_SVC
	svc_unregister(PIKTCPROG, PIKTCVERS);
	if (rmlocks(FALSE, TRUE) == ERROR)
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in sig_term(), rmlocks() error");
	#else
	if (isexecwait)
		free_execcmdout(hostcurr, FALSE);
	if (rmlocks(TRUE, TRUE) == ERROR)
		logreturn(PIKTLOG, LOG_ERR, __FILE__, __LINE__, FALSE, "in sig_term(), rmlocks() error");
	if (isdiff) {
		sprintf(cmd, "%s %s/* 2>/dev/null", RMCMD, difdir);
		system(cmd);
	}
	sprintf(cmd, "%s %s/* 2>/dev/null", RMCMD, stgdir);
	system(cmd);
	#endif
	exit(0);
}
