%{

/*
   cmd.l -- PIKT pikt command-line lexer

   $Header: /home/cvs/ucgsb/pikt/src/piktd/cmd.l,v 1.19 2000/02/14 01:33:54 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2000 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
 */

#include "../lib/common.h"
#include "piktd.h"
#include "cmd.tab.h"

#undef YY_INPUT
#define YY_INPUT(buf, result, max_size)	{ \
		signed char c = cmdinput(); \
		result = (c == EOF) ? YY_NULL : (buf[0] = c, 1); \
	}

char errormsg[MAXLINE+1];

void cmderror(const char *);
signed int cmdinput(void);

%}

W		[ \t]
D		[0-9]
L		[A-Za-z]
DL		[0-9A-Za-z]

%x mS pS

%option noyywrap

%%

"-"				{ BEGIN(mS);
				  return _MINUS;
				}
"+"				{ BEGIN(pS);
				  return _PLUS;
				}

<mS>h				{ ishelp = TRUE;
				  return _OPT;
				}
<mS>G				{ isdebug = TRUE;
				  return _OPT;
				}
<mS>T				{ istest = TRUE;
				  isdebug = TRUE;
				  return _OPT;
				}

<mS>c				{ ischeck = TRUE;
				  return _CHECK;
				}
<mS>C				{ isdoublecheck = TRUE;
				  istest = TRUE;
				  return _DOUBLECHECK;
				}

<pS>M				{ return _MAILCMD; }
<pS>L				{ return _LPCMD; }

<pS>A|S				{ return _ALERT; /* _SCRIPT */ }

<mS,pS,INITIAL>{W}		{ BEGIN(INITIAL); }

<mS,pS>.			{ sprintf(errormsg, "illegal option: %c", yytext[0]);
				  cmderror(errormsg);
				  ishelp = TRUE;
				  return _OPT;
				 }

[^-+][^+]*[^+ \t\n]		{ cmdlval.sval = mystrdup(yytext, "cmdlex");
				  return _ITEM;
				}

%%

/* cmd error handler
 */
void
cmderror (const char *message)
{

	fprintf(stderr, "%s (in cmdparse()): %s\n", message, yytext ? yytext : "");
	fflush(stderr);
	bailout = TRUE;
	return;
}

/* feed command-line arguments to the lexer input stream
 */
signed int
cmdinput (void)
{
	char c;

	if (argvcurr > argvlast)
		return(EOF);
	if ((c = (*argvcurr)[stroffset++]) != '\0')
		return(c);
	/* end of argument, move to the next */
	argvcurr++;
	stroffset = 0;
	return(' ');
}
